//
//  GameView.m
//  PhysicsDemo
//
//  Created by Christoph Wick on 25/09/14.
//  Copyright (c) 2014 Christoph Wick, i4innovation GmbH. All rights reserved.
//

#import "GameView.h"

@implementation GameView
{

	SCNNode *carNode;
	SCNPhysicsVehicle *carPhysics;
	float acceleration;
	float steeringAngle;
	float maxSteeringAngle;
}
-(void) awakeFromNib {
	acceleration = 0;
	steeringAngle = 0;
	maxSteeringAngle = 0.6;
}

-(void)mouseDown:(NSEvent *)theEvent
{
	NSLog(@"Got mouse down ...");
	// check what nodes are clicked
	NSPoint p = [self convertPoint:[theEvent locationInWindow] fromView:nil];
	NSArray *hitResults = [self hitTest:NSPointToCGPoint(p) options:nil];

	// check that we clicked on at least one object
	if([hitResults count] > 0){

		// retrieved the first clicked object
		for (SCNHitTestResult *result in hitResults) {

			SCNNode *resultNode = result.node;
			
			if ([resultNode.name isEqual: @"moreBallsNode"]) {
				[self throwMoreBalls];
			} else { // look in all parent nodes
				while (resultNode.parentNode != nil) {
					resultNode = resultNode.parentNode;
					if ([resultNode.name isEqual: @"moreBallsNode"]) {
						[self throwMoreBalls];
					}
				}
				
			}
			
		}
	}
	[super mouseDown:theEvent];
}


- (void)keyDown:(NSEvent *)theEvent
{
	
	NSLog(@"Got a keyDown event with %@", theEvent);
	[super keyDown:theEvent];

	
	if (theEvent.keyCode==49) { // space-key
		[self throwMoreBalls];
	}
	
	if (theEvent.keyCode==126) { // up-key
		acceleration -= 3;
		[carPhysics applyEngineForce:acceleration forWheelAtIndex:2];
		[carPhysics applyEngineForce:acceleration forWheelAtIndex:3];
	}
	if (theEvent.keyCode==125) { // down-key
		acceleration += 3;
		[carPhysics applyEngineForce:acceleration forWheelAtIndex:2];
		[carPhysics applyEngineForce:acceleration forWheelAtIndex:3];
	}
	if ([theEvent.characters  isEqual: @"a"]) { // a-key
		acceleration = 0;
		[carPhysics applyEngineForce:0 forWheelAtIndex:2];
		[carPhysics applyEngineForce:0 forWheelAtIndex:3];
		[carPhysics applyBrakingForce:3 forWheelAtIndex:2];
		[carPhysics applyBrakingForce:3 forWheelAtIndex:3];
	}
	
	if (theEvent.keyCode==124) { // right-key
		steeringAngle -= 0.05;
		if (steeringAngle < -1*maxSteeringAngle) {
			steeringAngle = -1*maxSteeringAngle;
		}
		[carPhysics setSteeringAngle:steeringAngle forWheelAtIndex:0];
		[carPhysics setSteeringAngle:steeringAngle forWheelAtIndex:1];
	}
	if (theEvent.keyCode==123) { // left key
		steeringAngle += 0.05;
		if (steeringAngle > maxSteeringAngle) {
			steeringAngle = maxSteeringAngle;
		}
		[carPhysics setSteeringAngle:steeringAngle forWheelAtIndex:0];
		[carPhysics setSteeringAngle:steeringAngle forWheelAtIndex:1];
	}
	if ([theEvent.characters  isEqual: @"b"]) { // be-key
		NSLog(@"Show balls");
		[self showBallsText];
	}
	
	if (theEvent.keyCode==36) { // return-key
		[self addCar];
	}
}

-(void) throwMoreBalls {
	
	for (int i=0; i<20; i++) {
		SCNSphere *sphereGeometry = [SCNSphere sphereWithRadius:2.0];
		sphereGeometry.firstMaterial.diffuse.contents = [NSImage imageNamed:@"billard.png"];
		SCNNode *sphereNode = [SCNNode nodeWithGeometry:sphereGeometry];
		
		int x=(arc4random_uniform(2)-1);
		int y=(arc4random_uniform(10)+20);
		int z=(arc4random_uniform(2)-1);
		
		//NSLog(@"Got vector: %i,%i,%i", x,y,z);
		
		sphereNode.position = SCNVector3Make(x,y,z);
		sphereNode.physicsBody = [SCNPhysicsBody dynamicBody];
		sphereNode.physicsBody.mass = 10;
		sphereNode.physicsBody.velocity = SCNVector3Make(0, -15, 0);
		
		[self.scene.rootNode addChildNode:sphereNode];
	}
}



- (void) showBallsText {
	
	for (SCNNode *childNode in self.scene.rootNode.childNodes) {
		if ([childNode.name  isEqual: @"moreBallsNode"]) {
			if (childNode.position.y == 30) {
				childNode.position = SCNVector3Make(childNode.position.x, 1, childNode.position.z);
			}
		}
	}
	
	
}

-(void) addCar {
	
	if (carNode == nil) {
	
		carNode = [SCNNode node];
		
		carNode.name = @"theCar";
		
		SCNBox *carBodyBox = [SCNBox boxWithWidth:5.0 height:3 length:10.0 chamferRadius:0.0];
		carBodyBox.firstMaterial.diffuse.contents = [NSColor grayColor];
		SCNNode *carBodyNode = [SCNNode nodeWithGeometry:carBodyBox];
		carBodyNode.position = SCNVector3Make(0.0, 2.8, 0.0);
		[carNode addChildNode:carBodyNode];
		
		SCNCylinder *wheel = [SCNCylinder cylinderWithRadius:1.5 height:0.8];
		wheel.firstMaterial.diffuse.contents = [NSColor darkGrayColor];
		
		SCNNode *lfWheel = [SCNNode node];
		lfWheel.position = SCNVector3Make(-3.0, 1.5, -3);
		SCNNode *lfWheelInner = [SCNNode nodeWithGeometry:wheel];
		lfWheelInner.rotation = SCNVector4Make(0.0, 0.0, 1.0, M_PI_2);
		[lfWheel addChildNode:lfWheelInner];
		[carNode addChildNode:lfWheel];
		
		SCNNode *rfWheel = [SCNNode node];
		rfWheel.position = SCNVector3Make(3.0, 1.5, -3);
		SCNNode *rfWheelInner = [SCNNode nodeWithGeometry:wheel];
		rfWheelInner.rotation = SCNVector4Make(0.0, 0.0, 1.0, M_PI_2);
		[rfWheel addChildNode:rfWheelInner];
		[carNode addChildNode:rfWheel];
		
		SCNNode *lrWheel = [SCNNode node];
		lrWheel.position = SCNVector3Make(-3.0, 1.5, 3);
		SCNNode *lrWheelInner = [SCNNode nodeWithGeometry:wheel];
		lrWheelInner.rotation = SCNVector4Make(0.0, 0.0, 1.0, M_PI_2);
		[lrWheel addChildNode:lrWheelInner];
		[carNode addChildNode:lrWheel];
		
		SCNNode *rrWheel = [SCNNode node];
		rrWheel.position = SCNVector3Make(3.0, 1.5, 3);
		SCNNode *rrWheelInner = [SCNNode nodeWithGeometry:wheel];
		rrWheelInner.rotation = SCNVector4Make(0.0, 0.0, 1.0, M_PI_2);
		[rrWheel addChildNode:rrWheelInner];
		[carNode addChildNode:rrWheel];
		
		
		SCNPhysicsBody *body = [SCNPhysicsBody dynamicBody];
		body.mass = 80;
		carNode.physicsBody = body;
		
		
		SCNPhysicsVehicleWheel *lfPhysicsWheel = [SCNPhysicsVehicleWheel wheelWithNode:lfWheel];
		SCNPhysicsVehicleWheel *rfPhysicsWheel = [SCNPhysicsVehicleWheel wheelWithNode:rfWheel];
		SCNPhysicsVehicleWheel *lrPhysicsWheel = [SCNPhysicsVehicleWheel wheelWithNode:lrWheel];
		SCNPhysicsVehicleWheel *rrPhysicsWheel = [SCNPhysicsVehicleWheel wheelWithNode:rrWheel];
		
		carPhysics = [SCNPhysicsVehicle vehicleWithChassisBody:carNode.physicsBody wheels:@[lfPhysicsWheel, rfPhysicsWheel,lrPhysicsWheel, rrPhysicsWheel]];

		[self.scene.physicsWorld addBehavior:carPhysics];
		[self.scene.rootNode addChildNode:carNode];
		
	}
}
@end
